Method: Stringex::StringExtensions::PublicInstanceMethods#to_html
- Defined in:
- lib/stringex/string_extensions.rb
#to_html(lite_mode = false) ⇒ Object
Returns the string converted (via Textile/RedCloth) to HTML format or self [with a friendly warning] if Redcloth is not available.
Using :lite argument will cause RedCloth to not wrap the HTML in a container P element, which is useful behavior for generating header element text, etc. This is roughly equivalent to ActionView’s textilize_without_paragraph except that it makes RedCloth do all the work instead of just gsubbing the return from RedCloth.
135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 |
# File 'lib/stringex/string_extensions.rb', line 135 def to_html(lite_mode = false) if defined?(RedCloth) if lite_mode RedCloth.new(self, [:lite_mode]).to_html else if self =~ /<pre>/ RedCloth.new(self).to_html.tr("\t", "") else RedCloth.new(self).to_html.tr("\t", "").gsub(/\n\n/, "") end end else warn "String#to_html was called without RedCloth being successfully required" self end end |